function [TP, EP] = Matrix2QuaternionComparison(samples)
%
% This function evaluates the performace, both in time and accuracy, of 
% the methods included in the folder 'Methods' for the computation of
% Euler parameters. 
%   
%  'samples' is the number of random orientations for which all methods 
%            are evaluated.
%  'TP'      is an array with the time performances.
%  'EP'      is an arrya with the error performances.
%


num_methods = 13;

for i=1:samples 
    e = single(RandomEulerParameters());
    e = [-0.0852  +0.7507   +0.0266   -0.6545];
    R = Euler2Matrix(e);
    [E, T(i,:)] = AllMethods(R);
    for j=1:num_methods
        error = abs(e)-abs(E(j,:));
        Y(i,j) = sum(error.^2);
    end
end

for i=1:num_methods
    W(i) = max(T(:,i));               % Worst-case time
    Z(i) = sum(T(:,i))/samples;       % Average time
    Y(i) = min(T(:,i));               % Best-case time 
    N(i) = samples - nnz(Y(:,i));     % Number of iterations without error
    M(i) = max(Y(:,i).^0.5);          % Maximum error
    A(i) = sum(Y(:,i).^0.5)/samples;  % Average error
    S(i) = sqrt(sum(Y(:,i))/samples); % Standard deviation
end

for i=1:num_methods
    TP(i,:) = [W(i) Z(i) Y(i)];       % Time performance
    EP(i,:) = [N(i) M(i) A(i) S(i)];  % Error performance
end
end
