 function [e, time]= BarItzhack_1(R, method)
%
% This function takes as input 'R', a proper rotation matrix, and returns the 
% corresponding Euler parameters 'e' using the method presented in:
%
% I. Bar-Itzhack, "An algorithm for computing the quaternion from the 
% rotation matrix," Journal of Guidance, Vol 23, No. 26, pp. 1085-1087, 2000. 
%

tic;

K = [ R(1,1)+R(2,2)    R(3,2)          -R(3,1)          R(2,1)-R(1,2); 
      R(3,2)           R(1,1)-R(2,2)    R(2,1)+R(1,2)   R(3,1);
     -R(3,1)           R(2,1)+R(1,2)    R(2,2)-R(1,1)   R(3,2);
      R(2,1)-R(1,2)    R(3,1)           R(3,2)         -R(1,1)-R(2,2)];


if method==1 % Method based on SVD 
    e = transpose(null(K-2*eye(4)));
end

if method==2 % Method based on Gaussian elimination
    M = rref([K-2*eye(4) eye(4)]);
    e = M(4,5:8);
    e = e/norm(e);
end
    
if e(1)<0
    e = -e;
end

time = toc;
end

