function [e, time] = ChiaveriniSiciliano(R)
%
% This function takes as input 'R', a proper rotation matrix, and returns 
% the corresponding Euler parameters 'e' using the formulas reported in:
%
% Chiaverini and B. Siciliano, "The unit quaternion: a useful tool for 
% inverse kinematics of robot manipulators," Systems Analysis 
% Modelling Simulation, Vol. 35, no. 1, pp. 45-60, 1999.
%

tic;

rad1 = 1+R(1,1)+R(2,2)+R(3,3);
rad2 = 1+R(1,1)-R(2,2)-R(3,3);
rad3 = 1-R(1,1)+R(2,2)-R(3,3);
rad4 = 1-R(1,1)-R(2,2)+R(3,3);

if rad1<0||rad2<0||rad3<0||rad4<0
    e(1) = NaN;
    e(2) = NaN;
    e(3) = NaN;
    e(4) = NaN;
else
    e(1) = sqrt(rad1)/2;
    e(2) = Mysign(R(3,2)-R(2,3))*sqrt(rad2)/2;
    e(3) = Mysign(R(1,3)-R(3,1))*sqrt(rad3)/2;
    e(4) = Mysign(R(2,1)-R(1,2))*sqrt(rad4)/2;
end

time=toc;

end






