function [e, time] = Coope(R, method)
%
% This function takes as input 'R', a proper rotation matrix, and returns 
% the corresponding Euler parameters 'e' according to the method presented
% in:
%
% I.D. Coope, A.B. Lincott, G.R. Dunlop, and M.I. Vuskovic, "Numerically 
% stable methods for converting rotation matrices to Euler parameters," 
% in  Advances in Robot Kinematics, J. Lenarcic and M. M. Stanisic (eds.), 
% Springer, 2000. 
%

tic;

threshold = 0.7;

radical = 1+R(1,1)+R(2,2)+R(3,3);

if radical<threshold
    e(1) = (1/(4*sqrt(1-radical/4)))* ...
            norm([R(2,3)-R(3,2) R(3,1)-R(1,3) R(1,2)-R(2,1)]);
else
    e(1) = sqrt(radical)/2;
end

if method==1 % Method based on SVD
    [V,D] = eig(R + R');
    eigenvalue = 0;
    index=0;
    for i=1:3
        if D(i,i)> eigenvalue
            eigenvalue = D(i,i);
            index = i;
        end
    end
    v = V(:,index);
end

if method==2 % Method based on Gaussian elimination 
    M = rref([(R - eye(3)) eye(3)]);
    v = M(3,4:6);
    v = v/norm(v);
end

e(2) = sqrt(1-radical/4)*v(1);
e(3) = sqrt(1-radical/4)*v(2);
e(4) = sqrt(1-radical/4)*v(3);

e(2) = Mysign(R(3,2)-R(2,3))*abs(e(2));
e(3) = Mysign(R(1,3)-R(3,1))*abs(e(3));
e(4) = Mysign(R(2,1)-R(1,2))*abs(e(4));

time = toc;
end
