function [e, time] = Hughes(R)
%
% This function takes as input 'R', a proper rotation matrix, and returns 
% the corresponding Euler parameters 'e' using the method described in:
% 
% P.C. Hughes, Spacecraft Attitude Dynamics, John Whiler & Sons, 1986.
%

tic;

radical = 1+R(1,1)+R(2,2)+R(3,3);

if radical <0
    e(1)= NaN;
    e(2)= NaN;
    e(3)= NaN;
    e(4)= NaN;
else
    if radical == 0
        e(1) = 0;
        e(2) = sqrt((1+R(1,1))/2);
        e(3) = sqrt((1+R(2,2))/2);
        e(4) = sqrt((1+R(3,3))/2);
        if R(1,2)<0 && R(2,3)<0
            e(3)= -e(3);
        end
        if R(2,3)<0 && R(3,1)<0
            e(4)= -e(4);
        end
        if R(3,1)<0 && R(1,2)<0
            e(2)= -e(2);
        end
    else
        e(1) = sqrt(radical)/2;
        e(2) = (R(3,2)-R(2,3))/(4*e(1));
        e(3) = (R(1,3)-R(3,1))/(4*e(1));
        e(4) = (R(2,1)-R(1,2))/(4*e(1));
    end
end

time = toc;
end

