function [e, time] = SarabandiThomas(R)
%
% This function takes as input 'R', a proper rotation matrix, and returns 
% the corresponding Euler parameters 'e' using the method presented in:
%
% S. Sarabandi and F. Thomas, "On the Computation of Euler Parameters
% from Rotation Matrices," Proc. of the 16th International Symposium on 
% Advances in Robot Kinematics, Bologna, Italy., 1-4 July, 2018..
%

tic;

threshold = 0.5;

rad(1) = 1+R(1,1)+R(2,2)+R(3,3);
rad(2) = 1+R(1,1)-R(2,2)-R(3,3);
rad(3) = 1-R(1,1)+R(2,2)-R(3,3);
rad(4) = 1-R(1,1)-R(2,2)+R(3,3);

if rad(1) < threshold
%    fprintf('%s\n','Improving e0');
    e(1) = 0.5*sqrt(((R(3,2)-R(2,3))^2+(R(1,3)-R(3,1))^2+(R(2,1)-R(1,2))^2)/(4-rad(1)));
else
    e(1) = sqrt(rad(1))/2;
end

if rad(2) < threshold
%    fprintf('%s\n','Improving e1');
    e(2) = 0.5*sqrt(((R(3,2)-R(2,3))^2+(R(1,2)+R(2,1))^2+(R(3,1)+R(1,3))^2)/(4-rad(2)));
else
    e(2) = sqrt(rad(2))/2;
end

if rad(3) < threshold
%    fprintf('%s\n','Improving e2');
    e(3) = 0.5*sqrt(((R(1,3)-R(3,1))^2+(R(1,2)+R(2,1))^2+(R(2,3)+R(3,2))^2)/(4-rad(3)));
else
    e(3) = sqrt(rad(3))/2;
end

if rad(4) < threshold
%    fprintf('%s\n','Improving e3');
    e(4) = 0.5*sqrt(((R(2,1)-R(1,2))^2+(R(3,1)+R(1,3))^2+(R(3,2)+R(2,3))^2)/(4-rad(4)));
else 
    e(4) = sqrt(rad(4))/2;
end

e(2) = Mysign(R(3,2)-R(2,3))*abs(e(2));
e(3) = Mysign(R(1,3)-R(3,1))*abs(e(3));
e(4) = Mysign(R(2,1)-R(1,2))*abs(e(4));

time = toc;

end

