function [e, time] = Shepperd(R)
%
% This function takes as input 'R', a proper rotation matrix, and returns the 
% corresponding Euler parameters 'e' using Shepperds Method. For details,
% see
%
% S.W. Sheppard, Quaternion from rotation matrix, Journal of Guidance
% and Control, Vol. 1, No. 3, pp. 223-224, 1978.
%

tic;

C=[1+R(1,1)+R(2,2)+R(3,3) R(3,2)-R(2,3)          R(1,3)-R(3,1)          R(2,1)-R(1,2); 
   R(3,2)-R(2,3)          1+R(1,1)-R(2,2)-R(3,3) R(1,2)+R(2,1)          R(3,1)+R(1,3);
   R(1,3)-R(3,1)          R(1,2)+R(2,1)          1-R(1,1)+R(2,2)-R(3,3) R(2,3)+R(3,2);
   R(2,1)-R(1,2)          R(3,1)+R(1,3)          R(2,3)+R(3,2)          1-R(1,1)-R(2,2)+R(3,3)];

v = [(R(1,1)+R(2,2)+R(3,3)) R(1,1) R(2,2) R(3,3)];

[~,index] = max(v);

big = C(index,index);

e = C(:,index)'/(2*sqrt(big));

if e(1)<0
    e= -e;
end

time = toc;

end
