function [e, time] = Trigonometric_1(R)
%
% This function takes as input R, a proper rotation matrix, and returns the 
% corresponding Euler parameters e using the first trigonometric method
% described in:
% 
% S. Sarabandi and F. Thomas, "A Survey on the Computation of Quaternions
% from Rotation Matrices,"  ASME Journal of Mechanisms and Robotics, 2018.
%

tic;

theta  = acos(R(3,3));
phi    = atan2(R(1,3),-R(2,3));
psi    = atan2(R(3,1),R(3,2));
psiphi = atan2(-R(1,2),R(1,1));

if theta<pi && theta>0
    e(1) = (cos(theta/2))*cos((phi+psi)/2);
    e(2) = (sin(theta/2))*cos((phi-psi)/2);
    e(3) = (sin(theta/2))*sin((phi-psi)/2);
    e(4) = (cos(theta/2))*sin((phi+psi)/2);
else if theta <= 0
        e(1) = cos(psiphi/2);
        e(2) = 0;
        e(3) = 0;
        e(4) = sin(psiphi/2);
    else if theta >= pi
            e(1) = 0;
            e(2) = cos(psiphi/2);
            e(3) = sin(psiphi/2);
            e(4) = 0;
        end
    end
end

if e(1)<0
    e = -e;
end

time = toc;

end

