function [e, time] = Trigonometric_2(R)
%
% This function takes as input R, a proper rotation matrix, and returns the 
% corresponding Euler parameters e using the method described in: 
%
% R.P. Paul, Robot Manipulators: Mathematics, Programming, and Control, 
% MIT Press Cambridge, MA, 1982.
%

tic;

s2 = sqrt(((R(3,2)-R(2,3))^2)+((R(1,3)-R(3,1))^2)+((R(2,1)-R(1,2))^2));
c2 = R(1,1)+R(2,2)+R(3,3)-1;
theta = atan2(s2,c2);

m(1) = Mysign(R(3,2)-R(2,3))*sqrt((R(1,1)-c2/2)/(1-c2/2));
m(2) = Mysign(R(1,3)-R(3,1))*sqrt((R(2,2)-c2/2)/(1-c2/2));
m(3) = Mysign(R(2,1)-R(1,2))*sqrt((R(3,3)-c2/2)/(1-c2/2));
 
big = max(m);

n(1) = Mysign(R(3,2)-R(2,3))*m(1);
n(2) = Mysign(R(1,3)-R(3,1))*m(2);
n(3) = Mysign(R(2,1)-R(1,2))*m(3);

if big <= m(1)
    ee(2) = n(1);
    ee(3) = (R(2,1)+R(1,2))/(2*n(1)*(1-c2/2));
    ee(4) = (R(1,3)+R(3,1))/(2*n(1)*(1-c2/2));
elseif big <= m(2)
    ee(2) = (R(2,1)+R(1,2))/(2*n(2)*(1-c2/2));
    ee(3) = n(2);
    ee(4) = (R(1,3)+R(3,1))/(2*n(2)*(1-c2/2));
elseif big <= m(3) 
    ee(2) = (R(2,3)+R(3,2))/(2*n(3)*(1-c2/2));
    ee(3) = (R(1,3)+R(3,1))/(2*n(3)*(1-c2/2));
    ee(4) = n(3);
end

if  pi>theta>pi/2
    e(1) = cos(theta/2);
    e(2) = ee(2)*sin(theta/2);
    e(3) = ee(3)*sin(theta/2);
    e(4) = ee(4)*sin(theta/2);
else
    e(1) = cos(theta/2);
    e(2) = (R(3,2)-R(2,3))*sin(theta/2)/s2;
    e(3) = (R(1,3)-R(3,1))*sin(theta/2)/s2;
    e(4) = (R(2,1)-R(1,2))*sin(theta/2)/s2;
end

time = toc;

end
